package jus.apcoo.tda4;

import jus.util.geometrie.*;
/**
 * Modlise la notion de coordonne avec une reprsentation cartsienne
 * @author P.Morat ou http://imag.fr/Philippe.Morat
 * @version 1.0 date 1/10/00
 * @motcle point, coordonne, espace, cartsien
 * @see <a href="PointCartesien.java">PointCartesien</a>
 * @see Vecteur
 */
public class PointCartesien {
    /** La reprsentation physique des coordonnes */
    private double x,y;
    /**
     * Restitue l'abscisse du point
     * @return l'abscisse du point
     */
    public double abscisse() { return x;}
    /**
     * Restitue l'ordonne du point
     * @return l'ordonne du point
     */
    public double ordonnee() { return y;}
    /**
     * Construction d'un point  l'origine.
     * @ensure PointASaPlace : abscisse()==0 && ordonnee()==0
     */
    public PointCartesien(){positionner(0.0,0.0);}
    /**
     * Construction d'un point
     * @param x la premire coordonne (abscisse)
     * @param y la seconde coordonne (ordonne)
     * @ensure PointASaPlace : abscisse()==x && ordonnee()==y
     */
    public PointCartesien(double x, double y) {positionner(x,y);}
    /**
     * Construction d'un point par clonage
     * @param p le point  cloner
     * @require p existe : p!=null
     * @ensure PointASaPlace : abscisse()==p.abscisse() && ordonnee()==p.ordonnee()
     */
    public PointCartesien(PointCartesien p) {this(p.abscisse(),p.ordonnee());}
    /**
     * Restitue la reprsentation textuelle du point
     * @return la chaine
     */
    public String toString() {return"PointCartesien{"+x+","+y+"}";}
    /**
     * Positionne le point  la coordonne cartsienne {x,y}
     * @param x l'abscisse
     * @param y l'ordonne
     * @ensure PointASaPlace : abscisse()==x && ordonnee()==y
     */
    public void positionner(double x, double y) {this.x=x; this.y=y;}
    /**
     * Compare 2 points cartsiens
     * @param p l'autre point
     * @require PointValide : p!=null
     * @return true si les points sont gaux  1.0e-5 prs
     */
    public boolean equals(PointCartesien p) {
        double dx= x-p.abscisse(), dy= y-p.ordonnee();
        if(dx<0) dx=-dx; if(dy<0) dy=-dy;
        return (dx<1.0e-5 && dy<1.0e-5);
    }
    /**
     * Translate le point selon le vecteur v
     * @param v le vecteur de translation
     * @require VecteurValide : v!=null
     * @ensure TranslationOk : new Vecteur(_this,this).equals(v)
     */
    public void translation(Vecteur v) { positionner(x+v.dx(),y+v.dy()); }
    /**
     * Rotation du point par rapport  l'origine d'un angle a
     * @param a l'angle <b>en degr</b>
     */
    public void rotation(double a) { //  complter en travail  la maison ....
    }
    /**
     * Rotation du point par rapport  un point quelconque d'un angle a
     * @param p point servant de centre de rotation
     * @param a l'angle <b>en degr</b>
     * @require PointValide : p!=null
     */
    public void rotation(PointCartesien p, double a) {
        Vecteur v = new Vecteur(-p.abscisse(),-p.ordonnee());
        translation(v);          // changement de repre, translation ramenant p  l'origine
        rotation(a);             // rotation de a  l'origine
        translation(v.oppose()); // re-changement de repre, translation ramenant p  sa position initiale
    }
    
}// PointCartesien
