package jus.apcoo.tda4;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JApplet;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Container;
import javax.swing.JFrame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**	
* Applet proposant une Interface utilisateur pour la manipulation 
* d'une tortue  la Logo.
*
* Cette interface peut tre invoque :<BR>
* <UL>
*    <LI>soit comme une applet dans le contexte d'une page html
*    (charger par exemple la page testTurtle.htm avec un navigateur 
*    quip d'une machine virtuelle java 2, ou utiliser l'application
*    appletviewer).
*    </LI>
*    <LI>soit comme une application autonome (lance par la commande
*    java TestTurtle.
* </UL>
* 
* @author	P.Morat ou http://imag.fr/Philippe.Morat ...
*
* @version	1.0 date : 1/9/99
* @version      1.1 date : 26/10/2001 Philippe Genoud
*
* mots-cls : tortue, Logo, trac, dessin
* @see <a href="TestTurtle.java">testTurtle</a>
* @see TurtleArea
* @see Turtle
*/
public class TestTurtle extends JApplet {
  /**
   * La tortue controle par cette interface
   */
  private Turtle turtle;
  /**
   * la zone de dessin dans laquelle se dplace la tortue
   */
  private TurtleArea turtleArea;
  /**
   * La checkbox controlant la position de la plume
   */	
  private JCheckBox plumePosChckBox;
  /**
   * textField pour la saisie du dplacement
   */
  private JTextField distTxtF;
  /**
   * textField pour la saisie de l'angle
   */
  private JTextField angTxtF;	
  /**
   * Les classes reprsentant les actions associes aux commandes
   * de l'environnement "Turtle"
   */
  private class Avancer implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      double dist = Double.parseDouble(distTxtF.getText());
      turtle.avancer((int) dist);
    }
  }

  private class Reculer implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      double dist = Double.parseDouble(distTxtF.getText());
      turtle.reculer((int) dist);
    }
  }

  private class Droite implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      double theta = Double.parseDouble(angTxtF.getText());
      turtle.droite((int) theta);
    }
  }

  private class Gauche implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      double theta = Double.parseDouble(angTxtF.getText());
      turtle.gauche((int)theta);
    }
  }

  private class Plume implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      if(plumePosChckBox.isSelected())
      	turtle.lever();
      else
       	turtle.baisser();
    }
  }

  private class Clear implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      turtleArea.clear();
    }
  }

  private class Quit implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      System.exit(0);
    }
  }

  private static class Exit extends WindowAdapter {
    public void windowClosing(WindowEvent e) {
      System.exit(0);
    }
  }

  /**
   *  initialise l'interface graphique en mode applet
   */
  public void init() {
    buildUI(this.getContentPane());
  }
  /**
   * cration et positionnement des diffrents composants de l'interface
   * utilisateur.
   *
   * @param container Le Container qui accueille les diffrents composants de
   *         l'interface
   *
   * RequiresArgumentOk : container!=null
   */
  private void buildUI(Container container) {
    //-------------------------------------------------------------------
    // un vertical Box Layout est associ au container
    // les composants ajouts  ce container seront poistionns les uns 
    // en dessous des autres dans leur ordre d'ajout
    //-------------------------------------------------------------------
    container.setLayout(new BoxLayout(container,BoxLayout.Y_AXIS));
    //-------------------------------------------------------------------
    // le premier composant ajout est la "turtleArea", zone dans laquelle
    // se dplace la tortue
    //-------------------------------------------------------------------
    turtleArea = new TurtleArea();
    turtle = new Turtle(turtleArea);
    turtleArea.addTurtle(turtle);
    container.setLayout(new BorderLayout());
    container.add(turtleArea,BorderLayout.CENTER);
    //-------------------------------------------------------------------
    // le deuxime composant est un JPanel dans lequel sont disposs
    // les textfields pour la saisie de la distance de dplacement et
    // de l'angle de rotation de la tortue
    //-------------------------------------------------------------------
    JPanel pane0 = new JPanel();
    pane0.setLayout(new GridLayout(2,0));
    JPanel pane1 = new JPanel();
    pane1.setBorder(BorderFactory.createEmptyBorder(30,30,10,30));
    pane1.setLayout(new FlowLayout());
    distTxtF = new JTextField("10",5);
    angTxtF = new JTextField("0",4);
    pane1.add(new JLabel("Distance"));
    pane1.add(distTxtF);
    pane1.add(new JLabel("Angulation"));
    pane1.add(angTxtF);
    pane0.add(pane1);
    //-------------------------------------------------------------------
    // le troisime composant est un autre JPanel dans lequel sont 
    // placs les diffrents boutons contrlant le dplacement de la
    // tortue ainsi que la checkBox contrlant la position de la plume
    //-------------------------------------------------------------------
    JPanel pane2 = new JPanel();
    pane2.setBorder(BorderFactory.createEmptyBorder(30,30,10,30));
    pane2.setLayout(new FlowLayout());
    // Les JButtons controlant les dplacements de la tortue
    JButton avancerBtn = new JButton("Avancer");
    avancerBtn.addActionListener(new Avancer());
    JButton reculerBtn = new JButton("Reculer");
    reculerBtn.addActionListener(new Reculer());
    JButton droiteBtn = new JButton("Droite");
    droiteBtn.addActionListener(new Droite());
    JButton gaucheBtn = new JButton("Gauche");
    gaucheBtn.addActionListener(new Gauche());
    JButton quitBtn = new JButton("Quit");
    quitBtn.addActionListener(new Quit());
    JButton clearBtn = new JButton("Clear");
    clearBtn.addActionListener(new Clear());
    // la checkBox controlant la position de la plume
    plumePosChckBox = new JCheckBox("Is Up",true);
    plumePosChckBox.addActionListener(new Plume());
    // ajout de ces composants au deuxime panel
    pane2.add(avancerBtn);
    pane2.add(reculerBtn);
    pane2.add(droiteBtn);
    pane2.add(gaucheBtn);
    pane2.add(plumePosChckBox);
    pane2.add(clearBtn);
    pane2.add(quitBtn);
    // ajout du panel au container
    pane0.add(pane2);
    container.add(pane0,BorderLayout.SOUTH);
  }
  /**
   *  excution en mode application autonome (stand-alone application).
   *  l'application cre une fentre dans laquelle l'interface est affiche
   */
  public static void main(String[] args) {
    // cration de la fentre principale de l'application
    JFrame f = new JFrame("TestTurtle");
    f.addWindowListener(new Exit());
    // cration de l'interface
    TestTurtle testturtle = new TestTurtle();
    testturtle.buildUI(f.getContentPane());
    // calcul automatique de la taille de la fentre en fonction de
    // son contenu et des layout managers utiliss
    f.pack();
    // affichage de la fentre
    // et lancement du thread de gestion des vnements
    f.setVisible(true);
  }
}













